<?php

// Experimental Zend API replacement for i18n. Just a draft.

class I18nExtension
{
    private $listI18n;

    public function __construct()
    {
        $this->listI18n = [];
    }

    public function i18n_new($locale, $domain)
    {
        $i18n = i18n_new($domain, $locale);

        if (!$i18n) {
            trigger_error("i18n_new did not return a handle", E_WARNING);
            return 0;
        }

        $index = zend_list_insert($i18n, $this->listI18n);
        $this->listI18n[] = $index;
        return $index;
    }

    public function i18n_availlocales($domain)
    {
        $result = i18n_availlocales($domain);

        if ($result === false) {
            trigger_error("Could not initialize array", E_ERROR);
            return false;
        }

        return $result;
    }

    public function i18n_locales($i18nIndex, $domain)
    {
        $i18n = $this->getI18nHandle($i18nIndex);

        if (!$i18n) {
            trigger_error("$i18nIndex is not a valid i18n object index!", E_WARNING);
            return false;
        }

        $result = i18n_locales($i18n, $domain);

        if ($result === false) {
            trigger_error("Could not initialize array", E_ERROR);
            return false;
        }

        return $result;
    }

    public function i18n_get_property($i18nIndex, $property, $domain, $language = null)
    {
        $i18n = $this->getI18nHandle($i18nIndex);

        if (!$i18n) {
            trigger_error("$i18nIndex is not a valid i18n object index!", E_WARNING);
            return false;
        }

        if ($language !== null) {
            $result = i18n_get_property($i18n, $property, $domain, $language);
        } else {
            $result = i18n_get_property($i18n, $property, $domain, null);
        }

        return $result;
    }

    public function i18n_get_file($i18nIndex, $file)
    {
        $i18n = $this->getI18nHandle($i18nIndex);

        if (!$i18n) {
            trigger_error("$i18nIndex is not a valid i18n object index!", E_WARNING);
            return false;
        }

        $result = i18n_get_file($i18n, $file);
        return $result;
    }

    public function i18n_interpolate($i18nIndex, $magicStr, $vars = [])
    {
        $i18n = $this->getI18nHandle($i18nIndex);

        if (!$i18n) {
            trigger_error("$i18nIndex is not a valid i18n object index!", E_WARNING);
            return false;
        }

        $i18nVars = $this->hashToVars($vars);
        $result = i18n_interpolate($i18n, $magicStr, $i18nVars);

        return $result;
    }

    // ... Implementations for other functions ...

    private function getI18nHandle($i18nIndex)
    {
        $type = null;
        $i18n = zend_list_find($i18nIndex, $type);

        if (!$i18n) {
            trigger_error("$i18nIndex is not a valid i18n object index!", E_WARNING);
            return false;
        }

        return $i18n;
    }


    public function __destruct()
    {
        $this->php_i18n_close();
    }

    private function php_i18n_close()
    {
        if ($this->listI18n) {
            foreach ($this->listI18n as $i18n) {
                i18n_destroy($i18n);
            }
        }
    }

    private function hashToVars($hash)
    {
        $vars = i18n_vars_new();

        if (!$hash) {
            return $vars;
        }

        foreach ($hash as $key => $value) {
            i18n_vars_add($vars, $key, $value);
        }

        return $vars;
    }
}
?>